"""
Extention to the OpenMetadata Workflow class
"""
from metadata.ingestion.api.workflow import Workflow
from metadata.generated.schema.metadataIngestion.workflow import (
    OpenMetadataWorkflowConfig,
)
from prefect import get_run_logger


class PrefectOpenMetadataIngestion(Workflow):
    """
    OpenMetadata ingestion workflow that adds a method
    allowing to log the workflow status to the Prefect backend.

    Args:
         config: string with a JSON configuration file
    """

    def __int__(self, config: OpenMetadataWorkflowConfig):
        """
        Args:
            config: string with a JSON configuration file
        """
        super().__init__(config=config)

    def log_flow_status(self) -> None:
        """
        Log workflow status to the Prefect API backend
        """
        logger = get_run_logger()
        logger.info("Source Status: %s", self.source.get_status().as_string())
        if hasattr(self, "stage"):
            logger.info("Stage Status: %s", self.stage.get_status().as_string())
        if hasattr(self, "sink"):
            logger.info("Sink Status: %s", self.sink.get_status().as_string())
        if hasattr(self, "bulk_sink"):
            logger.info("Bulk Sink Status: %s", self.bulk_sink.get_status().as_string())



"""
## Prefect flow for metadata ingestion

Follow the main documentation for guidance on:

- installing and configuring `Prefect` and `OpenMetadata`,
- running metadata ingestion flows locally and on schedule.
"""
import json
from prefect import flow
from prefect_openmetadata.ingestion_workflow import PrefectOpenMetadataIngestion


@flow
def ingest_metadata(config: str) -> None:
    """
    Args:
        config: JSON-formatted configuration

    Examples:
        Flow ingesting metadata using Prefect:
        ```python
        from prefect_openmetadata.flows import ingest_metadata

        json_config = \"""See an example in the section: Run ingestion flow\"""

        if __name__ == "__main__":
            ingest_metadata(json_config)
        ```
    """
    workflow_config = json.loads(config)
    workflow = PrefectOpenMetadataIngestion.create(workflow_config)
    workflow.execute()
    workflow.raise_from_status()
    workflow.log_flow_status()
    workflow.stop()
